/*
** Module   : Menu.java
** Abstract : Menu abstract client implementation
**
** Copyright (c) 2015, Golden Code Development Corporation.
** ALL RIGHTS RESERVED. Use is subject to license terms.
**
**           Golden Code Development Corporation
**                      CONFIDENTIAL
**
** -#- -I- --Date-- -----------------------------Description-----------------------------
** 001 VIG 20150116 Created initial version.
** 002 VIG 20150226 Added method getFirstEnabledWidget.
*/
package com.goldencode.p2j.ui.client;

import com.goldencode.p2j.ui.*;
import com.goldencode.p2j.ui.client.event.KeyInput;
import com.goldencode.p2j.ui.client.widget.*;

/**
 * <code>Menu</code> client abstract implementation class. 
 */
public abstract class Menu<O extends OutputManager<?>>
extends AbstractContainer<O>
{
   /** Menu attribute container. */
   private MenuConfig config = null; 

   /**
    * Special constructor used to restore component using given configuration.
    * 
    * @param   id
    *          The widget's ID.
    * @param   cfg
    *          Configuration data.
    */
   public Menu(WidgetId id, MenuConfig cfg)
   {
   }
   
   /**
    * Replacement API for the logic which should exist in the widget's c'tor, but the logic 
    * can't be executed as it depends on the widget being registered into the official registry.
    * <p>
    * This API is called after the widget is created and added to the registry.
    * 
    * @param    id
    *           The widget's ID.
    * @param    cfg
    *           The config used to initialize this widget.
    */
   @Override
   public void initialize(WidgetId id, WidgetConfig cfg)
   {
      config = ConfigManager.getInstance().getActiveConfig(id);
      if (config == null)
      {
         config = new MenuConfig(id);
      }
      ConfigManager.getInstance().replaceConfig(config, cfg);
   }   

   /**
    * Provide access to the attributes stored in the widget.
    * 
    * @return  Reference to attribute container.
    */
   @SuppressWarnings("unchecked")
   @Override
   public MenuConfig config()
   {
      return config;
   }
   
   /**
    * Set displayed flag of this menu.
    * 
    * @param   displayed
    *          <code>true</code> means that menu is dropped down, 
    *          <code>false</code> otherwise.
    */
   public void setDisplayed(boolean displayed)
   {
      if (config.popupOnly)
      {
         config.displayed = displayed;
      }
   }

   /**
    * Check if widget is displayed.
    * 
    * @return  <code>true</code> if widget is displayed.
    */
   @Override
   public boolean isDisplayed()
   {
      if (config.popupOnly)
      {
         return config.displayed;
      }

      Widget<?> owner = screen().getRegistry().getComponent(config.ownerId);
      
      return owner == null ? false : owner.isDisplayed();
   }

   /**
    * Process key event.
    *
    * @param   event
    *          Key event to process.
    */
   @Override
   public void processKeyEvent(KeyInput event)
   {
      super.processKeyEvent(event);
   }
   
   /**
    * Looks up the very first enabled widget in specified frames.
    * 
    * @return  the first enabled widget or <code>null</code>
    */
   public Widget<O> getFirstEnabledWidget()
   {
      // find the first enabled widget of this frame
      Widget<O>[] widgets = widgets();
   
      for (Widget<O> comp : widgets)
      {
         if (UiUtils.hasConfig(comp) && comp.focusTraversable())
         {
            return comp;
         }
      }
   
      // no enabled widgets in all frames
      return null; 
   }
}
