/*
** Module   : Rectangle.java
** Abstract : Rectangular area descriptor.
**
** Copyright (c) 2010-2014, Golden Code Development Corporation.
** ALL RIGHTS RESERVED. Use is subject to license terms.
**
**           Golden Code Development Corporation
**                      CONFIDENTIAL
**
** -#- -I- --Date-- ---------------------------------Description----------------------------------
** 001 SIY 20100629 Created initial version
** 002 SIY 20100804 Changes in imports, some refactorings.
** 003 CA  20140729 Added size, difference and hashCode APIs.
** 004 HC  20140911 GUI coordinate support - data types refactoring.
** 005 HC  20140924 Hynek placeholder text
** 006 HC  20141021 Implemented window move/position support.
*/

package com.goldencode.p2j.ui.client;

import com.goldencode.p2j.ui.Coordinate;

/**
 * Rectangular area descriptor.
 * All coordinate values are in character units. 
 */
public class Rectangle
{
   /** Rectangle bottom row. */
   private  double bottom;

   /** Rectangle left column. */
   private  double left;

   /** Rectangle right column. */
   private  double right;

   /** Rectangle top row. */
   private  double top;

   /**
    * Constructor.
    * 
    * @param   top
    *          Top row.
    * @param   left
    *          Left column.
    * @param   bottom
    *          Bottom row.
    * @param   right
    *          Right column.
    */
   public Rectangle(double top, double left, double bottom, double right)
   {
      this.top = Coordinate.scale(top);
      this.left = Coordinate.scale(left);
      this.bottom = Coordinate.scale(bottom);
      this.right = Coordinate.scale(right);
   }

   /**
    * Constructor.
    * 
    * @param   topleft
    *          Top-left corner.
    * @param   size
    *          Rectangle dimension.
    */
   public Rectangle(Point topleft, Dimension size)
   {
      top = topleft.y;
      left = topleft.x;
      
      // TODO: #2323 The base coordinate value differs for ChUI and GUI modes.
      //       Once Coordinate.isChui() is implemented, fix this as well.
      //       Until then do not use this constructor in GUI.
      bottom = Coordinate.scale(top + size.height - 1);
      right = Coordinate.scale(left + size.width - 1);
   }

   /**
    * Copy constructor.
    * 
    * @param   rect
    *          Source rectangle.
    */
   public Rectangle(Rectangle rect)
   {
      top = rect.top();
      left = rect.left();
      bottom = rect.bottom();
      right = rect.right();
   }

   /**
    * Get bottom coordinate of this rectangle.
    * 
    * @return  bottom.
    */
   public double bottom()
   {
      return bottom;
   }

   /**
    * Returns <code>true</code> if the specified point is inside this
    * rectangle.
    * 
    * @param   x
    *          X coordinate.
    * @param   y
    *          Y coordinate.
    * 
    * @return <code>true</code> if coordinates are inside rectangle.
    */
   public boolean contains(double x, double y)
   {
      return ((x >= left) && (x <= right) && (y >= top) && (y <= bottom));
   }

   /**
    * Returns <code>true</code> if the specified point is inside this
    * rectangle.
    * 
    * @param   p
    *          Point to check.
    *          
    * @return  <code>true</code> if specified point is inside rectangle.
    */
   public boolean contains(Point p)
   {
      return contains(p.x, p.y);
   }

   /**
    * Returns <code>true</code> is specified rectangle is completely inside
    * this rectangle.
    * 
    * @param   r
    *          Rectangle to check.
    *          
    * @return  <code>true</code> if specified rectangle is inside this
    *          rectangle.
    */
   public boolean contains(Rectangle r)
   {
      return ((r.left >= left) && (r.right <= right) && (r.top >= top) && (r.bottom <= bottom));
   }

   /**
    * Check if rectangle is empty.
    * 
    * @return  <code>true</code> if rectangle is empty.
    */
   public boolean empty()
   {
      return top > bottom || right < left;
   }

   /**
    * Check if this rectangle is equal to specified one.
    * 
    * @param   rect
    *          Rectangle to check.
    * 
    * @return  <code>true</code> if rectangles are equal.
    */
   public boolean equals(Rectangle rect)
   {
      if (rect == null)
         return false;
      if (top != rect.top)
         return false;
      if (bottom != rect.bottom)
         return false;
      if (left != rect.left)
         return false;
      if (right != rect.right)
         return false;
      return true;
   }
   
   /**
    * Compute the hashcode of this rectangle.
    * 
    * @return   A hashcode uniquely describing this rectangle.
    */
   @Override
   public int hashCode()
   {
      final int prime = 31;
      int result = 1;
      long temp;
      temp = Double.doubleToLongBits(bottom);
      result = prime * result + (int) (temp ^ (temp >>> 32));
      temp = Double.doubleToLongBits(left);
      result = prime * result + (int) (temp ^ (temp >>> 32));
      temp = Double.doubleToLongBits(right);
      result = prime * result + (int) (temp ^ (temp >>> 32));
      temp = Double.doubleToLongBits(top);
      result = prime * result + (int) (temp ^ (temp >>> 32));
      return result;
   }

   @Override
   public boolean equals(Object obj)
   {
      if (this == obj)
         return true;
      if (obj == null)
         return false;
      if (getClass() != obj.getClass())
         return false;
      Rectangle other = (Rectangle) obj;
      if (Double.doubleToLongBits(bottom) != Double.doubleToLongBits(other.bottom))
         return false;
      if (Double.doubleToLongBits(left) != Double.doubleToLongBits(other.left))
         return false;
      if (Double.doubleToLongBits(right) != Double.doubleToLongBits(other.right))
         return false;
      if (Double.doubleToLongBits(top) != Double.doubleToLongBits(other.top))
         return false;
      return true;
   }

   /**
    * Calculate rectangle which is intersection between this and specified
    * rectangle.
    * 
    * @param   rect
    *          Source rectangle.
    *          
    * @return  rectangle which is an intersection between this and specified
    *          rectangle or <code>null</code> if there is no intersection.
    */
   public Rectangle intersection(Rectangle rect)
   {
      if (top > rect.bottom)
         return null;
      if (bottom < rect.top)
         return null;
      if (left > rect.right)
         return null;
      if (right < rect.left)
         return null;

      return new Rectangle((top > rect.top) ? top
                                           : rect.top,
                           (left > rect.left) ? left
                                             : rect.left,
                           (bottom < rect.bottom) ? bottom
                                                 : rect.bottom,
                           (right < rect.right) ? right
                                               : rect.right);
   }

   /**
    * Check if this and specified rectangle have intersection.
    * 
    * @param   rect
    *          Source rectangle.
    * 
    * @return  <code>true</code> if rectangles have intersection.
    */
   public boolean intersects(Rectangle rect)
   {
      if (rect.left > right)
         return false;
      if (rect.right < left)
         return false;
      if (rect.top > bottom)
         return false;
      if (rect.bottom < top)
         return false;
      return true;
   }

   /**
    * Get left coordinate of this rectangle.
    * 
    * @return  left.
    */
   public double left()
   {
      return left;
   }

   /**
    * Get right coordinate of this rectangle.
    * 
    * @return  right.
    */
   public double right()
   {
      return right;
   }

   /**
    * Get top coordinate of this rectangle.
    * 
    * @return top.
    */
   public double top()
   {
      return top;
   }

   /**
    * Create new rectangle which is translated by specified offset.
    * 
    * @param   dx
    *          Delta X.
    * @param   dy
    *          Delta Y.
    * 
    * @return  translated rectangle.
    */
   public Rectangle translate(double dx, double dy)
   {
      return new Rectangle(top + dy, left + dx, bottom + dy, right + dx);
   }

   /**
    * Create new rectangle which is translated by specified offset.
    * 
    * @param   p
    *          Offset represented as a point.
    *           
    * @return  translated rectangle.
    */
   public Rectangle translate(Point p)
   {
      return translate(p.x, p.y);
   }

   /**
    * Create new rectangle which is a union of this and specified rectangle.
    * 
    * @param   rect
    *          Source rectangle.
    * 
    * @return  union rectangle.
    */
   public Rectangle union(Rectangle rect)
   {
      return new Rectangle((top < rect.top) ? top
                                           : rect.top,
                           (left < rect.left) ? left
                                             : rect.left,
                           (bottom > rect.bottom) ? bottom
                                                 : rect.bottom,
                           (right > rect.right) ? right
                                               : rect.right);
   }
}
