/*
** Module   : MenuChuiImpl.java
** Abstract : CHUI client implementation of Menu.
**
** Copyright (c) 2015, Golden Code Development Corporation.
** ALL RIGHTS RESERVED. Use is subject to license terms.
**
**           Golden Code Development Corporation
**                      CONFIDENTIAL
**
** -#- -I- --Date-- -----------------------------Description-----------------------------
** 001 VIG 20150116 Created initial version.
** 002 VIG 20150226 Added processKeyEvent method.
*/
package com.goldencode.p2j.ui.client.chui;

import com.goldencode.p2j.ui.*;
import com.goldencode.p2j.ui.client.*;
import com.goldencode.p2j.ui.client.border.*;
import com.goldencode.p2j.ui.client.chui.driver.ChuiOutputManager;
import com.goldencode.p2j.ui.client.event.*;
import com.goldencode.p2j.ui.client.widget.*;

/**
 * <code>Menu</code> CHUI client implementation class. 
 */
@SuppressWarnings("unchecked")
public class MenuChuiImpl
extends Menu<ChuiOutputManager>
{
   /** Border around the panel. */
   private Border<ChuiOutputManager> border = null;
   
   /**
    * Special constructor used to restore component using given configuration.
    * 
    * @param   id
    *          The widget's ID.
    * @param   cfg
    *          Configuration data.
    */
   public MenuChuiImpl(WidgetId id, WidgetConfig cfg)
   {
      super(id, (MenuConfig) cfg);
      
      MenuConfig mcfg = (MenuConfig) cfg;
      if (mcfg.popupOnly)
      {
         border = new LineBorder();
      }
   }
   
   /**
    * Drawing worker method.
    */
   public void drawPanel()
   {
      screen().clear(this);

      if (border != null)
         border.drawBorder(this, dimension());
   }

   /**
    * Draw the peer container. The methods calls the draw() method for
    * ScrollPane instance for the peer container. In addition the required
    * clipping rectangle is calculated for ChildFrame containers because they
    * must be rendered inside the parent container.
    * <p>
    * Drawing for the redirected terminal differs. Labels/headers are drawn
    * only once per page, instead of the usual once per menu.
    */
   @Override
   public void draw()
   {
      drawPanel();
      super.draw();
   }

   /**
    * Hide/show widget.
    * <p>
    * If the menu is being shown and not yet attached to a window, attached it to the default
    * window.
    * 
    * @param   visible
    *          New visible state.
    */
   @Override
   public void setVisible(boolean visible)
   {
      setDisplayed(visible);
      super.setVisible(visible);
      
      if (visible && parent() == null)
      {
         setParent((Container<ChuiOutputManager>) Window.getDefaultWindow());
      }
      
      for(Widget<ChuiOutputManager> w : widgets())
      {
         w.setVisible(true);
      }

   }

   /**
    * Replace original version to take into account P2J specific behavior.
    * 
    * @see  com.goldencode.p2j.ui.client.widget.Widget#dimension()
    */
   @Override
   public Dimension dimension()
   {
      double height = SubMenuChuiImpl.getMaxHeight(this);
      
      if (config().popupOnly)
      {
         double width = 0;  // maximum item width
         for(Widget<?> w : widgets())
         {
            AbstractWidget<?> aw = (AbstractWidget<?>) w;
            if (width < aw.width())
            {
               width = aw.width(); 
            }
         }
         return new Dimension(width + 2, height); //widgets().length + 2
      }
      else
      {
         double width = 0; 
         for(Widget<?> w : widgets())
         {
            AbstractWidget<?> aw = (AbstractWidget<?>) w;
            width += aw.width() + 1; 
         }
         return new Dimension(width + 2, height); // 1
      }
   }

   /**
    * Process key event.
    *
    * @param   event
    *          Key event to process.
    */
   @Override
   public void processKeyEvent(KeyInput event)
   {
      if (event.isConsumed())
      {
         return;
      }

      // mnemonic 
      if (event.keyChar() != KeyInput.CHAR_UNDEFINED)
      {
         char ch = Character.toLowerCase(event.keyChar());
         for(Widget<?> w : getWidgetsList())
         {
            if (w instanceof MenuItem)
            {
               MenuItem<?> m = (MenuItem<?>) w;
               if (m.config().mnemonic != null && // mnemonic 
                   Character.toLowerCase(m.config().mnemonic.charValue()) == ch) 
               {
                  EventManager.postEvent(new ItemEvent(m));
                  return;
               }
            }
         }
      }
      // accelerator
      else if (event.isRealKey())
      {
         int code = event.keyCode();
         for(Widget<?> w : getWidgetsList())
         {
            if (w instanceof MenuItem)
            {
               MenuItem<?> m = (MenuItem<?>) w;
               if (m.config().accelerator != null && 
                   Keyboard.keyCode(m.config().accelerator) == code)
  
               {
                  EventManager.postEvent(new ItemEvent(m));
                  return;
               }
            }
         }
         
      }
      else
      {
         super.processKeyEvent(event);
      }
   }
   
   /**
    * Lay out container.
    */
   @Override
   public void doLayout()
   {
      setLocation(calcLocation());
      super.doLayout();
      
      Widget<ChuiOutputManager>[] w = widgets();
      double x=0;
      
      for(int i=0; i<w.length; i++)
      {
         if (!config().popupOnly) // menubar
         {
            w[i].setLocation(new Point(x, 0));
            x += 1 + w[i].width();
         }
         else
         {
            w[i].setLocation(new Point(1, i + 1));
         }
         
         if(w[i] instanceof SubMenu)
         {
            ((SubMenu<?>)w[i]).doLayout();
         }
      }
   }
   
   private Point calcLocation()
   {
      if (config().popupOnly)
      {
         Widget<?> owner = screen().getRegistry().getComponent(config().ownerId);
         if (owner != null)
         {
            AbstractWidget<?> awOwner = (AbstractWidget<?>) owner;
            return new Point(awOwner.location().x + 1, awOwner.location().y + awOwner.height() + 1);
         }
      }
      
      return new Point(0, 0); // menubar or owner is not set
   }
}
